@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -18,11 +18,11 @@
 #ifndef __ARCHINFO_H__
 #define __ARCHINFO_H__
 
-#include "archive.h"
-#include "dico.h"
+struct s_dico;
+struct s_archreader;
 
-int archinfo_show_mainhead(carchive *ai, cdico *dicomainhead);
-int archinfo_show_fshead(cdico *dicofshead, int fsid);
+int archinfo_show_mainhead(struct s_archreader *ai, struct s_dico *dicomainhead);
+int archinfo_show_fshead(struct s_dico *dicofshead, int fsid);
 char *compalgostr(int algo);
 char *cryptalgostr(int algo);
 
