@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -15,31 +15,35 @@
  * Homepage: http://www.fsarchiver.org
  */
 
-#include "fsarchiver.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
-#include <ext2fs/ext2fs.h>
-#include <blkid/blkid.h>
-#include <e2p/e2p.h>
+#include <ext2fs.h>
+#include <blkid.h>
+#include <e2p.h>
+#include <uuid.h>
 
+#include "fsarchiver.h"
 #include "dico.h"
 #include "common.h"
 #include "strlist.h"
-#include "uuid.h"
 #include "filesys.h"
 #include "fs_ext2.h"
+#include "error.h"
 
 // e2fsprogs version required to work on ext2, ext3, ext4
 u64 e2fsprogs_minver[]={PROGVER(1,39,0), PROGVER(1,39,0), PROGVER(1,41,0)};
 
 struct mntopt
-{      unsigned int    mask;
-        const char      *string;
+{   unsigned int   mask;
+    const char     *string;
 };
 
 struct s_features
 {
-    char    *name; // name of that feature
+    char   *name; // name of that feature
     int    mask; // identifier for that feature
     int    compat; // compat type for that feature (see e2p.h)
     int    firstfs; // type of the first filesystem to support it
@@ -49,17 +53,17 @@ struct s_features
 // TODO: check the real mke2fs version that supports the features
 struct s_features mkfeatures[] = // cf e2fsprogs-1.41.4/lib/e2p/feature.c
 {
-    {"has_journal",        FSA_EXT3_FEATURE_COMPAT_HAS_JOURNAL,        E2P_FEATURE_COMPAT,        EXTFSTYPE_EXT3,        PROGVER(1,39,0)},
-    {"resize_inode",    FSA_EXT2_FEATURE_COMPAT_RESIZE_INODE,        E2P_FEATURE_COMPAT,        EXTFSTYPE_EXT2,        PROGVER(1,39,0)},
-    {"dir_index",        FSA_EXT2_FEATURE_COMPAT_DIR_INDEX,        E2P_FEATURE_COMPAT,        EXTFSTYPE_EXT2,        PROGVER(1,33,0)},
-    {"filetype",        FSA_EXT2_FEATURE_INCOMPAT_FILETYPE,        E2P_FEATURE_INCOMPAT,        EXTFSTYPE_EXT2,        PROGVER(1,16,0)},
-    {"extent",        FSA_EXT4_FEATURE_INCOMPAT_EXTENTS,        E2P_FEATURE_INCOMPAT,        EXTFSTYPE_EXT4,        PROGVER(1,41,0)},
-    {"journal_dev",        FSA_EXT3_FEATURE_INCOMPAT_JOURNAL_DEV,        E2P_FEATURE_INCOMPAT,        EXTFSTYPE_EXT3,        PROGVER(1,39,0)},
-    {"flex_bg",        FSA_EXT4_FEATURE_INCOMPAT_FLEX_BG,        E2P_FEATURE_INCOMPAT,        EXTFSTYPE_EXT4,        PROGVER(1,41,0)},
-    {"large_file",        FSA_EXT2_FEATURE_RO_COMPAT_LARGE_FILE,        E2P_FEATURE_RO_INCOMPAT,    EXTFSTYPE_EXT2,        PROGVER(1,40,7)},
-    {"sparse_super",    FSA_EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER,    E2P_FEATURE_RO_INCOMPAT,    EXTFSTYPE_EXT2,        PROGVER(1,8,0)},
-    {"uninit_bg",        FSA_EXT4_FEATURE_RO_COMPAT_GDT_CSUM,        E2P_FEATURE_RO_INCOMPAT,    EXTFSTYPE_EXT4,        PROGVER(1,41,0)},
-    {NULL,            0,                        0,                0},
+    {"has_journal",  FSA_EXT3_FEATURE_COMPAT_HAS_JOURNAL,     E2P_FEATURE_COMPAT,      EXTFSTYPE_EXT3, PROGVER(1,39,0)},
+    {"resize_inode", FSA_EXT2_FEATURE_COMPAT_RESIZE_INODE,    E2P_FEATURE_COMPAT,      EXTFSTYPE_EXT2, PROGVER(1,39,0)},
+    {"dir_index",    FSA_EXT2_FEATURE_COMPAT_DIR_INDEX,       E2P_FEATURE_COMPAT,      EXTFSTYPE_EXT2, PROGVER(1,33,0)},
+    {"filetype",     FSA_EXT2_FEATURE_INCOMPAT_FILETYPE,      E2P_FEATURE_INCOMPAT,    EXTFSTYPE_EXT2, PROGVER(1,16,0)},
+    {"extent",       FSA_EXT4_FEATURE_INCOMPAT_EXTENTS,       E2P_FEATURE_INCOMPAT,    EXTFSTYPE_EXT4, PROGVER(1,41,0)},
+    {"journal_dev",  FSA_EXT3_FEATURE_INCOMPAT_JOURNAL_DEV,   E2P_FEATURE_INCOMPAT,    EXTFSTYPE_EXT3, PROGVER(1,39,0)},
+    {"flex_bg",      FSA_EXT4_FEATURE_INCOMPAT_FLEX_BG,       E2P_FEATURE_INCOMPAT,    EXTFSTYPE_EXT4, PROGVER(1,41,0)},
+    {"large_file",   FSA_EXT2_FEATURE_RO_COMPAT_LARGE_FILE,   E2P_FEATURE_RO_INCOMPAT, EXTFSTYPE_EXT2, PROGVER(1,40,7)},
+    {"sparse_super", FSA_EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER, E2P_FEATURE_RO_INCOMPAT, EXTFSTYPE_EXT2, PROGVER(1,8,0)},
+    {"uninit_bg",    FSA_EXT4_FEATURE_RO_COMPAT_GDT_CSUM,     E2P_FEATURE_RO_INCOMPAT, EXTFSTYPE_EXT4, PROGVER(1,41,0)},
+    {NULL,           0,                                       0,                       0,              0},
 };
 
 char *format_fstype(int fstype)
@@ -140,7 +144,9 @@ int extfs_mkfs(cdico *d, char *partition, int extfstype)
     u64 e2fstoolsver;
     int compat_type;
     u64 temp64;
+    int exitst;
     int ret=0;
+    int res;
     int i;
     
     // init    
@@ -149,7 +155,7 @@ int extfs_mkfs(cdico *d, char *partition, int extfstype)
     strlist_init(&strfeatures);
     
     // ---- check that mkfs is installed and get its version
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "%s -V", progname)!=0)
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, NULL, 0, "%s -V", progname)!=0)
     {   errprintf("%s not found. please install a recent e2fsprogs on your system or check the PATH.\n", progname);
         ret=-1;
         goto extfs_mkfs_cleanup;
@@ -254,10 +260,16 @@ int extfs_mkfs(cdico *d, char *partition, int extfstype)
         goto extfs_mkfs_cleanup;
     }
     
+    // ---- extended options
+    if (dico_get_u64(d, 0, FSYSHEADKEY_FSEXTEOPTRAIDSTRIDE, &temp64)==0)
+        strlcatf(options, sizeof(options), " -E stride=%ld ", (long)temp64);
+    if ((dico_get_u64(d, 0, FSYSHEADKEY_FSEXTEOPTRAIDSTRIPEWIDTH, &temp64)==0) && e2fstoolsver>=PROGVER(1,40,7))
+        strlcatf(options, sizeof(options), " -E stripe-width=%ld ", (long)temp64);
+    
     // ---- execute mke2fs
     msgprintf(MSG_VERB2, "exec: %s\n", command);
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "%s %s %s", progname, partition, options)!=0)
-    {   errprintf("command [%s] failed\n", command);
+    if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "%s %s %s", progname, partition, options)!=0 || exitst!=0)
+    {   errprintf("command [%s] failed with return status=%d\n", command, exitst);
         ret=-1;
         goto extfs_mkfs_cleanup;
     }
@@ -270,10 +282,16 @@ int extfs_mkfs(cdico *d, char *partition, int extfstype)
     if (dico_get_string(d, 0, FSYSHEADKEY_FSEXTDEFMNTOPT, buffer, sizeof(buffer))==0 && strlen(buffer)>0)
         strlcatf(options, sizeof(options), " -o %s ", buffer);
     
+    if (dico_get_u64(d, 0, FSYSHEADKEY_FSEXTFSCKMAXMNTCOUNT, &temp64)==0)
+        strlcatf(options, sizeof(options), " -c %ld ", (long)temp64);
+    
+    if (dico_get_u64(d, 0, FSYSHEADKEY_FSEXTFSCKCHECKINTERVAL, &temp64)==0)
+        strlcatf(options, sizeof(options), " -i %ldd ", (long)(temp64/86400L));
+    
     if (options[0])
     {
-        if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "tune2fs %s %s", partition, options)!=0)
-        {   errprintf("command [%s] failed\n", command);
+        if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "tune2fs %s %s", partition, options)!=0 || exitst!=0)
+        {   errprintf("command [%s] failed with return status=%d\n", command, exitst);
             ret=-1;
             goto extfs_mkfs_cleanup;
         }
@@ -282,8 +300,8 @@ int extfs_mkfs(cdico *d, char *partition, int extfstype)
         // http://article.gmane.org/gmane.comp.file-systems.ext4/11181
         if (extfstype==EXTFSTYPE_EXT4 && e2fstoolsver<PROGVER(1,41,4))
         {
-            if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "e2fsck -fy %s", partition)!=0)
-            {   errprintf("command [%s] failed\n", command);
+            if ( ((res=exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "e2fsck -fy %s", partition))!=0) || ((exitst!=0) && (exitst!=1)) )
+            {   errprintf("command [%s] failed with return status=%d\n", command, exitst);
                 ret=-1;
                 goto extfs_mkfs_cleanup;
             }
@@ -297,9 +315,10 @@ extfs_mkfs_cleanup:
 
 int extfs_getinfo(cdico *d, char *devname)
 {
+    struct fsa_ext2_sb *super;
     blk_t use_superblock=0;
     int use_blocksize=0;
-    const char *str;
+    char uuid[512];
     ext2_filsys fs;
     int origextfstype;
     char mntopt[1024];
@@ -313,35 +332,56 @@ int extfs_getinfo(cdico *d, char *devname)
     {   errprintf("ext2fs_open(%s) failed\n", devname);
         return -1;
     }
+    super=(struct fsa_ext2_sb *)fs->super;
     
     // --- label
     memset(label, 0, sizeof(label));
-    if (fs->super->s_volume_name[0])
+    if (super->s_volume_name[0])
     {   memset(label, 0, sizeof(label));
-        strncpy(label, fs->super->s_volume_name, sizeof(fs->super->s_volume_name));
+        strncpy(label, super->s_volume_name, sizeof(super->s_volume_name));
     }
     dico_add_string(d, 0, FSYSHEADKEY_FSLABEL, label);
     
     // ---- uuid
-    if ((str=e2p_uuid2str(fs->super->s_uuid))!=NULL)
-        dico_add_string(d, 0, FSYSHEADKEY_FSUUID, str);
+    /*if ((str=e2p_uuid2str(super->s_uuid))!=NULL)
+        dico_add_string(d, 0, FSYSHEADKEY_FSUUID, str);*/
+    memset(uuid, 0, sizeof(uuid));
+    uuid_unparse_lower((u8*)super->s_uuid, uuid);
+    dico_add_string(d, 0, FSYSHEADKEY_FSUUID, uuid);
+    msgprintf(MSG_DEBUG1, "extfs_uuid=[%s]\n", uuid); 
     
     // ---- block size
-    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTBLOCKSIZE, EXT2_BLOCK_SIZE(fs->super));
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTBLOCKSIZE, EXT2_BLOCK_SIZE(super));
     
     // ---- filesystem revision (good-old-rev or dynamic)
-    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTREVISION, fs->super->s_rev_level);
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTREVISION, super->s_rev_level);
     
     // ---- inode size
-    if (fs->super->s_rev_level >= EXT2_DYNAMIC_REV)
-        dico_add_u64(d, 0, FSYSHEADKEY_FSINODESIZE, fs->super->s_inode_size);
+    if (super->s_rev_level >= EXT2_DYNAMIC_REV)
+        dico_add_u64(d, 0, FSYSHEADKEY_FSINODESIZE, super->s_inode_size);
     else
         dico_add_u64(d, 0, FSYSHEADKEY_FSINODESIZE, EXT2_GOOD_OLD_INODE_SIZE); // Good old rev
     
+    // ---- extended options
+    if (super->s_raid_stride > 0)
+    {   dico_add_u64(d, 0, FSYSHEADKEY_FSEXTEOPTRAIDSTRIDE, super->s_raid_stride);
+        msgprintf(MSG_DEBUG1, "extfs_raid_stride: %u\n", super->s_raid_stride);
+    }
+    if (super->s_raid_stripe_width > 0)
+    {   dico_add_u64(d, 0, FSYSHEADKEY_FSEXTEOPTRAIDSTRIPEWIDTH, super->s_raid_stripe_width);
+        msgprintf(MSG_DEBUG1, "extfs_raid_stripe_width: %u\n", super->s_raid_stripe_width);
+    }
+    
+    // ---- fsck details: max_mount_count and check_interval
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFSCKMAXMNTCOUNT, max(super->s_max_mnt_count,0));
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFSCKCHECKINTERVAL, super->s_checkinterval);
+    msgprintf(MSG_DEBUG1, "extfs_max_mount_count: %ld\n", (long)max(super->s_max_mnt_count,0));
+    msgprintf(MSG_DEBUG1, "extfs_check_interval: %ld\n", (long)super->s_checkinterval);
+    
     // ---- default mount options
     memset(mntopt, 0, sizeof(mntopt));
     count=0;
-    mask=fs->super->s_default_mount_opts;
+    mask=super->s_default_mount_opts;
     if (mask & EXT3_DEFM_JMODE)
     {   strlcatf(mntopt, sizeof(mntopt), "%s", e2p_mntopt2string(mask & EXT3_DEFM_JMODE));
         count++;
@@ -360,21 +400,21 @@ int extfs_getinfo(cdico *d, char *devname)
     msgprintf(MSG_DEBUG1, "default mount options: [%s]\n", mntopt);
     
     // ---- filesystem features
-    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFEATURECOMPAT, (u64)fs->super->s_feature_compat);
-    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFEATUREINCOMPAT, (u64)fs->super->s_feature_incompat);
-    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFEATUREROCOMPAT, (u64)fs->super->s_feature_ro_compat);
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFEATURECOMPAT, (u64)super->s_feature_compat);
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFEATUREINCOMPAT, (u64)super->s_feature_incompat);
+    dico_add_u64(d, 0, FSYSHEADKEY_FSEXTFEATUREROCOMPAT, (u64)super->s_feature_ro_compat);
     
-    origextfstype=extfs_get_fstype_from_compat_flags((u64)fs->super->s_feature_compat, (u64)fs->super->s_feature_incompat, (u64)fs->super->s_feature_ro_compat);
+    origextfstype=extfs_get_fstype_from_compat_flags((u64)super->s_feature_compat, (u64)super->s_feature_incompat, (u64)super->s_feature_ro_compat);
     msgprintf(MSG_DEBUG1, "the filesystem type determined by the features is [%s]\n", format_fstype(origextfstype));
     
     // ---- check that fsarchiver is aware of all the filesystem features used on that filesystem
-    if (extfs_check_compatibility((u64)fs->super->s_feature_compat, (u64)fs->super->s_feature_incompat, (u64)fs->super->s_feature_ro_compat)!=0)
+    if (extfs_check_compatibility((u64)super->s_feature_compat, (u64)super->s_feature_incompat, (u64)super->s_feature_ro_compat)!=0)
     {   errprintf("this filesystem has ext{2,3,4} features which are not supported by this fsarchiver version.\n");
         return -1;
     }
     
     // ---- minimum fsarchiver version required to restore
-    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 5, 9, 0));
+    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 6, 4, 0));
             
     ext2fs_close(fs);
     
@@ -478,3 +518,49 @@ int extfs_get_reqmntopt(char *partition, cstrlist *reqopt, cstrlist *badopt)
     
     return 0;
 }
+
+u64 check_prog_version(char *prog)
+{
+    char stderrbuf[2048];
+    char command[2048];
+    char options[1024];
+    char temp1[1024];
+    char delims[]="\n\r";
+    char *saveptr;
+    char *result;
+    int foundversion;
+    int x, y, z;
+    
+    // init
+    memset(options, 0, sizeof(options));
+    memset(stderrbuf, 0, sizeof(stderrbuf));
+    
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, stderrbuf, sizeof(stderrbuf), "%s -V", prog)!=0)
+    {   errprintf("program %s was not found or has bad permissions.\n", prog);
+        return -1;
+    }
+    
+    foundversion=false;
+    result=strtok_r(stderrbuf, delims, &saveptr);
+    while (result != NULL && foundversion==false)
+    {   if ((memcmp(result, prog, strlen(prog))==0))
+            foundversion=true;
+        else
+            result = strtok_r(NULL, delims, &saveptr);
+    }
+    
+    if (foundversion==false)
+    {   errprintf("can't parse %s version number: no match\n", prog);
+        return 0;
+    }
+    
+    x=y=z=0;
+    sscanf(result, "%s %d.%d.%d", temp1, &x, &y, &z);
+    
+    if (x==0 && y==0)
+    {   errprintf("can't parse %s version number: x=y=0\n", prog);
+        return 0;
+    }
+    
+    return PROGVER(x,y,z);
+}
