@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -15,7 +15,9 @@
  * Homepage: http://www.fsarchiver.org
  */
 
-#include "fsarchiver.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -24,24 +26,27 @@
 #include <sys/mount.h>
 
 #include "fsarchiver.h"
+#include "devinfo.h"
 #include "dico.h"
 #include "common.h"
 #include "fs_ntfs.h"
 #include "filesys.h"
 #include "strlist.h"
+#include "error.h"
 
 int ntfs_mkfs(cdico *d, char *partition)
 {
     char command[2048];
     char buffer[2048];
     char options[2048];
+    int exitst;
     u64 temp64;
     u32 temp32;
     u16 temp16;
     
     // there is no option that just displays the version and return 0 in mkfs.ntfs
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "which mkfs.ntfs")!=0)
-    {   errprintf("mkfs.ntfs not found. please install ntfs-3g 2009.1.1AR.1 or more recent on your system or check the PATH.\n");
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, NULL, 0, "mkfs.ntfs")!=0)
+    {   errprintf("mkfs.ntfs not found. please install ntfsprogs-2.0.0 on your system or check the PATH.\n");
         return -1;
     }
     
@@ -56,7 +61,7 @@ int ntfs_mkfs(cdico *d, char *partition)
     if (dico_get_u32(d, 0, FSYSHEADKEY_NTFSCLUSTERSIZE, &temp32)==0)
         strlcatf(options, sizeof(options), " -c %ld ", (long)temp32);
     
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "mkfs.ntfs -f %s %s", partition, options)!=0)
+    if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "mkfs.ntfs -f %s %s", partition, options)!=0 || exitst!=0)
     {   errprintf("command [%s] failed\n", command);
         return -1;
     }
@@ -68,133 +73,57 @@ int ntfs_mkfs(cdico *d, char *partition)
     return 0;
 }
 
-int ntfs_read_label(int fd, char *labelbuf, int bufsize, struct s_ntfsinfo *info)
+int ntfs_getinfo(cdico *d, char *devname)
 {
-    u8 cFileRecord[4096]; // 1024 in most cases
-    u64 offset;
-    u16 nOffsetSequenceAttribute;
-    u8 *cData;
-    u32 dwAttribType;
-    u32 dwAttribLen;
-    bool bAttribResident;    
-    u8 *cDataResident;
-    u32 nAttrSize;
-    u64 i;
+    struct s_devinfo devinfo;
+    struct s_ntfsinfo info;
+    char bootsect[512];
+    int fd=-1;
     
-    // init
-    memset(labelbuf, 0, bufsize);
-    
-    // 1. read $Volume record
-    offset = ((u64) info->lcn_of_mft_data_attrib * info->bytes_per_cluster) + ((u64) (3 * info->file_record_size));
-    
-    if (lseek(fd, offset, SEEK_SET)!=offset)
-    {   errprintf("lseek(fd, offset=%lld, SEEK_SET) failed\n", (long long)offset);
+    if (((fd=open64(devname, O_RDONLY|O_LARGEFILE))<0) ||
+        (read(fd, bootsect, sizeof(bootsect))!=sizeof(bootsect)) ||
+        (close(fd)<0))
+    {   sysprintf("cannot open device or read bootsector on %s\n", devname);
         return -1;
     }
     
-    if ((read(fd, cFileRecord, info->file_record_size))!=info->file_record_size)
-    {   errprintf("cannot read file record\n");
-        return -2;
-    }
-    
-    // -------- decode the MFT File record
-    nOffsetSequenceAttribute = le16_to_cpu(*((u16*)(cFileRecord+0x14)));
-    cData = cFileRecord + nOffsetSequenceAttribute;
-    
-    do
-    {   // szData points to the beginning of an attribute
-        dwAttribType = le32_to_cpu(*((u32*)(cData)));
-        dwAttribLen = le32_to_cpu(*((u32*)(cData+4)));
-        bAttribResident = le8_to_cpu(*((u8*)(cData+8)))==0;
-        
-        if(dwAttribType == 0x60) // "volume_name"
-        {
-            if (bAttribResident == false)
-            {   errprintf("$volume_name attribute is not resident\n");
-                return -3;
-            }
-            
-            nAttrSize = le16_to_cpu(*((u16*)(cData+0x10)));
-            cDataResident = cData+le16_to_cpu(*((u16*)(cData+0x14)));
-            
-            for (i=0; (i < nAttrSize/2) && (i<bufsize-1); i++)
-                labelbuf[i] = cDataResident[2*i];
-        }
-        cData += dwAttribLen;
-    } while (dwAttribType != (u32)-1); // attribute list ends with type -1
-    
-    return 0;
-}
-
-int ntfs_getinfo(cdico *d, char *devname)
-{
-    struct s_ntfsinfo info;
-    char bootsect[16384];
-    char label[256];
-    int ret=0;
-    int fd=-1;
-    
-    if ((fd=open64(devname, O_RDONLY|O_LARGEFILE))<0)
-    {   ret=-1;
-        errprintf("cannot open(%s, O_RDONLY)\n", devname);
-        goto ntfs_getinfo_return;
-    }
-    
-    if (read(fd, bootsect, sizeof(bootsect))!=sizeof(bootsect))
-    {   ret=-1;
-        errprintf("cannot read the boot sector on %s\n", devname);
-        goto ntfs_getinfo_close;
-    }
-    
+    // check signature in the boot sector
     if (memcmp(bootsect+3, "NTFS", 4) != 0)
-    {   ret=-1;
-        errprintf("cannot find the ntfs signature on %s\n", devname);
-        goto ntfs_getinfo_close;
+    {   errprintf("cannot find the ntfs signature on %s\n", devname);
+        return -1;
+    }
+    
+    // get device label from common code in libbklid
+    if (get_devinfo(&devinfo, devname)!=0)
+    {   errprintf("get_devinfo(%s) failed\n", devname);
+        return -1;
     }
     
     info.bytes_per_sector = le16_to_cpu(*((u16*)(bootsect+0xB)));
     info.sectors_per_clusters = le8_to_cpu(*((u8*)(bootsect+0xD)));
-    info.total_sector_count = le64_to_cpu(*((u64*)(bootsect+0x28)));
-    info.lcn_of_mft_data_attrib = le64_to_cpu(*((u64*)(bootsect+0x30)));
     info.bytes_per_cluster = info.bytes_per_sector * info.sectors_per_clusters;
-    info.clusters_per_mft_record = *((s8*)(bootsect+0x40));
     info.uuid = le64_to_cpu(*((u64*)(bootsect+0x48)));
     
-    if (info.clusters_per_mft_record > 0)
-        info.file_record_size = info.clusters_per_mft_record * info.bytes_per_cluster;
-    else
-        info.file_record_size = 1 << (-info.clusters_per_mft_record);
-    
-    msgprintf(MSG_DEBUG1, "bytes_per_sector=[%lld]\n", (long long)info.bytes_per_sector);
-    msgprintf(MSG_DEBUG1, "sectors_per_clusters=[%lld]\n", (long long)info.sectors_per_clusters);
-    msgprintf(MSG_DEBUG1, "total_sector_count=[%lld]\n", (long long)info.total_sector_count);
-    msgprintf(MSG_DEBUG1, "clusters_per_mft_record=[%lld]\n", (long long)info.clusters_per_mft_record);
-    msgprintf(MSG_DEBUG1, "lcn_of_mft_data_attrib=[%lld]\n", (long long)info.lcn_of_mft_data_attrib);
-    msgprintf(MSG_DEBUG1, "bytes_per_cluster=[%lld]\n", (long long)info.bytes_per_cluster);
-    msgprintf(MSG_DEBUG1, "lcn_of_mft_data_attrib=[%lld]\n", (long long)info.lcn_of_mft_data_attrib);
-    msgprintf(MSG_DEBUG1, "file_record_size=[%lld]\n", (long long)info.file_record_size);
-    msgprintf(MSG_DEBUG1, "uuid=[%016llX]\n", (long long unsigned int)info.uuid);
+    msgprintf(MSG_VERB2, "bytes_per_sector=[%lld]\n", (long long)info.bytes_per_sector);
+    msgprintf(MSG_VERB2, "sectors_per_clusters=[%lld]\n", (long long)info.sectors_per_clusters);
+    msgprintf(MSG_VERB2, "bytes_per_cluster=[%lld]\n", (long long)info.bytes_per_cluster);
+    msgprintf(MSG_VERB2, "uuid=[%016llX]\n", (long long unsigned int)info.uuid);
     
     dico_add_u16(d, 0, FSYSHEADKEY_NTFSSECTORSIZE, info.bytes_per_sector);
     dico_add_u32(d, 0, FSYSHEADKEY_NTFSCLUSTERSIZE, info.bytes_per_cluster);
     dico_add_u64(d, 0, FSYSHEADKEY_NTFSUUID, info.uuid);
     
-    if (ntfs_read_label(fd, label, sizeof(label), &info)!=0)
-        snprintf(label, sizeof(label), "<unknonw>");
+    // get label from library
+    dico_add_string(d, 0, FSYSHEADKEY_FSLABEL, devinfo.label);
+    msgprintf(MSG_VERB2, "ntfs_label=[%s]\n", devinfo.label);
     
-    msgprintf(MSG_DEBUG1, "ntfs_label=[%s]\n", label);
-    dico_add_string(d, 0, FSYSHEADKEY_FSLABEL, label);
+    // minimum fsarchiver version required to restore
+    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 6, 4, 0));
     
-    // ---- minimum fsarchiver version required to restore
-    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 5, 9, 0));
-    
-    // ---- save mount options used at savefs so that restfs can use consistent mount options
+    // save mount options used at savefs so that restfs can use consistent mount options
     dico_add_string(d, 0, FSYSHEADKEY_MOUNTINFO, "streams_interface=xattr"); // may change in the future
     
-ntfs_getinfo_close:
-    close(fd);
-ntfs_getinfo_return:
-    return ret;
+    return 0;
 }
 
 int ntfs_mount(char *partition, char *mntbuf, char *fsbuf, int flags, char *mntinfo)
@@ -209,22 +138,21 @@ int ntfs_mount(char *partition, char *mntbuf, char *fsbuf, int flags, char *mnti
     char *saveptr;
     char *result;
     u64 instver=0;
+    int exitst;
     
     // init
     memset(options, 0, sizeof(options));
+    memset(stderrbuf, 0, sizeof(stderrbuf));
     snprintf(minversion, sizeof(minversion), "ntfs-3g %.4d.%.2d.%.2d (standard release)", 
         NTFS3G_MINVER_Y, NTFS3G_MINVER_M, NTFS3G_MINVER_D);
     
-    // check that mount.ntfs-3g is available
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "which mount.ntfs-3g")!=0)
-    {   errprintf("mount.ntfs-3g not found. please install %s\n"
+    // check that mount.ntfs-3g is available (don't check the exit status, it's not supposed to be 0)
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, stderrbuf, sizeof(stderrbuf), "ntfs-3g -h")!=0)
+    {   errprintf("ntfs-3g not found. please install %s\n"
             "or a newer version on your system or check the PATH.\n", minversion);
         return -1;
     }
     
-    memset(stderrbuf, 0, sizeof(stderrbuf));
-    exec_command(command, sizeof(command), NULL, 0, stderrbuf, sizeof(stderrbuf), "mount.ntfs-3g -h");
-    
     // check if there is a recent ntfs-3g version installed
     result=strtok_r(stderrbuf, delims, &saveptr);
     while (result != NULL && instver==0)
@@ -237,7 +165,7 @@ int ntfs_mount(char *partition, char *mntbuf, char *fsbuf, int flags, char *mnti
     
     if (instver < NTFS3G_VERSION(NTFS3G_MINVER_Y, NTFS3G_MINVER_M, NTFS3G_MINVER_D))
     {
-        errprintf("fsarchiver requires %s to operate. The version detected is too old\n", minversion);
+        errprintf("fsarchiver requires %s to operate. The detected version is too old\n", minversion);
         return -1;
     }
     else
@@ -262,7 +190,7 @@ int ntfs_mount(char *partition, char *mntbuf, char *fsbuf, int flags, char *mnti
         strlcatf(options, sizeof(options), " -o ro ");
     
     // ---- set the advanced filesystem settings from the dico
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "mount.ntfs-3g %s %s %s", options, partition, mntbuf)!=0)
+    if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "ntfs-3g %s %s %s", options, partition, mntbuf)!=0 || exitst!=0)
     {   errprintf("command [%s] failed, make sure a recent version of ntfs-3g is installed\n", command);
         return -1;
     }
@@ -273,13 +201,14 @@ int ntfs_mount(char *partition, char *mntbuf, char *fsbuf, int flags, char *mnti
 int ntfs_umount(char *partition, char *mntbuf)
 {
     char command[2048];
+    int existst;
 
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "which fusermount")!=0)
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, NULL, 0, "fusermount")!=0)
     {   errprintf("fusermount not found. please install fuse on your system or check the PATH.\n");
         return -1;
     }
     
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "fusermount -u %s", mntbuf)!=0)
+    if (exec_command(command, sizeof(command), &existst, NULL, 0, NULL, 0, "fusermount -u %s", mntbuf)!=0 || existst!=0)
     {   errprintf("cannot unmount [%s]\n", mntbuf);
         return -1;
     }
