@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -15,29 +15,34 @@
  * Homepage: http://www.fsarchiver.org
  */
 
-#include "fsarchiver.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <string.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <uuid.h>
 
+#include "fsarchiver.h"
 #include "dico.h"
 #include "common.h"
 #include "fs_xfs.h"
-#include "uuid.h"
 #include "filesys.h"
 #include "strlist.h"
+#include "error.h"
 
 int xfs_mkfs(cdico *d, char *partition)
 {
     char command[2048];
     char buffer[2048];
     char options[2048];
+    int exitst;
     u64 temp64;
     
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "mkfs.xfs -V")!=0)
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, NULL, 0, "mkfs.xfs -V")!=0)
     {   errprintf("mkfs.xfs not found. please install xfsprogs on your system or check the PATH.\n");
         return -1;
     }
@@ -49,7 +54,7 @@ int xfs_mkfs(cdico *d, char *partition)
     if ((dico_get_u64(d, 0, FSYSHEADKEY_FSXFSBLOCKSIZE, &temp64)==0) && (temp64%512==0) && (temp64>=512) && (temp64<=65536))
         strlcatf(options, sizeof(options), " -b size=%ld ", (long)temp64);
     
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "mkfs.xfs -f %s %s", partition, options)!=0)
+    if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "mkfs.xfs -f %s %s", partition, options)!=0 || exitst!=0)
     {   errprintf("command [%s] failed\n", command);
         return -1;
     }
@@ -61,7 +66,7 @@ int xfs_mkfs(cdico *d, char *partition)
     
     if (options[0])
     {
-        if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "xfs_admin %s %s", options, partition)!=0)
+        if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "xfs_admin %s %s", options, partition)!=0 || exitst!=-0)
         {   errprintf("command [%s] failed\n", command);
             return -1;
         }
@@ -73,7 +78,7 @@ int xfs_mkfs(cdico *d, char *partition)
 int xfs_getinfo(cdico *d, char *devname)
 {
     struct xfs_sb sb;
-    const char *str;
+    char uuid[512];
     u32 temp32;
     int ret=0;
     int fd;
@@ -102,9 +107,12 @@ int xfs_getinfo(cdico *d, char *devname)
     dico_add_string(d, 0, FSYSHEADKEY_FSLABEL, (char*)sb.sb_fname);
     
     // ---- uuid
-    if ((str=e2p_uuid2str(&sb.sb_uuid))!=NULL)
-        dico_add_string(d, 0, FSYSHEADKEY_FSUUID, str);
-    msgprintf(MSG_DEBUG1, "xfs_uuid=[%s]\n", str);
+    /*if ((str=e2p_uuid2str(&sb.sb_uuid))!=NULL)
+        dico_add_string(d, 0, FSYSHEADKEY_FSUUID, str);*/
+    memset(uuid, 0, sizeof(uuid));
+    uuid_unparse_lower((u8*)&sb.sb_uuid, uuid);
+    dico_add_string(d, 0, FSYSHEADKEY_FSUUID, uuid);
+    msgprintf(MSG_DEBUG1, "xfs_uuid=[%s]\n", uuid);
     
     // ---- block size
     temp32=be32_to_cpu(sb.sb_blocksize);
@@ -117,7 +125,7 @@ int xfs_getinfo(cdico *d, char *devname)
     msgprintf(MSG_DEBUG1, "xfs_blksize=[%ld]\n", (long)temp32);
     
     // ---- minimum fsarchiver version required to restore
-    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 5, 9, 0));
+    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 6, 4, 0));
     
 xfs_read_sb_close:
     close(fd);
