@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <attr/xattr.h>
 #include <sys/time.h>
+#include <sys/stat.h>
 #include <gcrypt.h>
 
 #include "fsarchiver.h"
@@ -1385,11 +1386,6 @@ int oper_restore(char *archive, int argc, char **argv, int oper)
         goto do_extract_error;
     }
     
-    if (oper==OPER_ARCHINFO && archinfo_show_mainhead(&exar.ai, dicomainhead)!=0)
-    {   errprintf("archinfo_show_mainhead(%s) failed\n", archive);
-        goto do_extract_error;
-    }
-    
     // check that the minimum fsarchiver version required is ok
     curver=FSA_VERSION_BUILD(PACKAGE_VERSION_A, PACKAGE_VERSION_B, PACKAGE_VERSION_C, PACKAGE_VERSION_D);
     if (exar.ai.minfsaver > 0)
@@ -1406,6 +1402,12 @@ int oper_restore(char *archive, int argc, char **argv, int oper)
         goto do_extract_error;
     }
     
+    // show archive information if command is OPER_ARCHINFO
+    if (oper==OPER_ARCHINFO && archinfo_show_mainhead(&exar.ai, dicomainhead)!=0)
+    {   errprintf("archinfo_show_mainhead(%s) failed\n", archive);
+        goto do_extract_error;
+    }
+    
     // check that the operation requested on the command line matches the archive type
     switch (exar.ai.archtype)
     {
