@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -15,7 +15,9 @@
  * Homepage: http://www.fsarchiver.org
  */
 
-#include "fsarchiver.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -26,39 +28,12 @@
 #include <errno.h>
 #include <assert.h>
 
+#include "fsarchiver.h"
 #include "queue.h"
+#include "dico.h"
 #include "common.h"
 #include "syncthread.h"
-
-#define printwait()                                                                \
-{                                                                       \
-    msgprintf(MSG_DEBUG4, "WAIT: %s() and qlen=%ld\n", __FUNCTION__, (long)q->itemcount);                            \
-}
-
-#define fct_in()                                                                \
-{                                                                       \
-    msgprintf(MSG_DEBUG5, "[thr=%.2ld] FCT_IN: %s() and qlen=%ld\n", (long)(pthread_self()%100), __FUNCTION__, (long)q->itemcount);        \
-}
-#define fct_out()                                                                \
-{                                                                       \
-    msgprintf(MSG_DEBUG5, "[thr=%.2ld] FCT_OUT: %s() and qlen=%ld\n", (long)(pthread_self()%100), __FUNCTION__, (long)q->itemcount);    \
-}
-
-char *qerr(s64 err)
-{
-    switch (err)
-    {
-        case QERR_SUCCESS: return "QERR_SUCCESS";
-        case QERR_FAIL: return "QERR_FAIL";
-        case QERR_NOMEM: return "QERR_NOMEM";
-        case QERR_INVAL: return "QERR_INVAL";
-        case QERR_NOTFOUND: return "QERR_NOTFOUND";
-        case QERR_ENDOFQUEUE: return "QERR_ENDOFQUEUE";
-        case QERR_WRONGTYPE: return "QERR_WRONGTYPE";
-        case QERR_CLOSED: return "QERR_CLOSED";
-        default: return "QERR_UNKNOWN";
-    }
-}
+#include "error.h"
 
 struct timespec get_timeout()
 {
@@ -74,7 +49,7 @@ s64 queue_init(cqueue *q, s64 blkmax)
 
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     // ---- init default attributes
@@ -90,25 +65,25 @@ s64 queue_init(cqueue *q, s64 blkmax)
     assert(pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK)==0);
     if (pthread_mutex_init(&q->mutex, &attr)!=0)
     {   msgprintf(3, "pthread_mutex_init failed\n");
-        return QERR_FAIL;
+        return FSAERR_UNKNOWN;
     }
     
     if (pthread_cond_init(&q->cond,NULL)!=0)
     {   msgprintf(3, "pthread_cond_init failed\n");
-        return QERR_FAIL;
+        return FSAERR_UNKNOWN;
     }
     
-    return QERR_SUCCESS;
+    return FSAERR_SUCCESS;
 }
 
 s64 queue_destroy(cqueue *q)
 {
-    struct s_queueitem *cur;
-    struct s_queueitem *next;
+    cqueueitem *cur;
+    cqueueitem *next;
     
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     assert(pthread_mutex_lock(&q->mutex)==0);
@@ -126,23 +101,21 @@ s64 queue_destroy(cqueue *q)
     assert(pthread_mutex_destroy(&q->mutex)==0);
     assert(pthread_cond_destroy(&q->cond)==0);
     
-    return QERR_SUCCESS;
+    return FSAERR_SUCCESS;
 }
 
 s64 queue_set_end_of_queue(cqueue *q, bool state)
 {
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
 
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     q->endofqueue=state;
     assert(pthread_mutex_unlock(&q->mutex)==0);
     pthread_cond_broadcast(&q->cond);
-    fct_out();
-    return QERR_SUCCESS;
+    return FSAERR_SUCCESS;
 }
 
 bool queue_get_end_of_queue(cqueue *q)
@@ -150,14 +123,12 @@ bool queue_get_end_of_queue(cqueue *q)
     bool res;
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
 
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     res=((q->itemcount<1) && (q->endofqueue==true));
     assert(pthread_mutex_unlock(&q->mutex)==0);
-    fct_out();
     return res;
 }
 
@@ -166,12 +137,10 @@ bool queuelocked_get_end_of_queue(cqueue *q)
     bool res;
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
 
-    fct_in();
     res=((q->itemcount<1) && (q->endofqueue==true));
-    fct_out();
     return res;
 }
 
@@ -182,14 +151,12 @@ s64 queue_count(cqueue *q)
     
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
 
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     itemcount = q->itemcount;
     assert(pthread_mutex_unlock(&q->mutex)==0);
-    fct_out();
     
     return itemcount;
 }
@@ -197,15 +164,14 @@ s64 queue_count(cqueue *q)
 // how many items in the queue have a particular status
 s64 queue_count_status(cqueue *q, int status)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     int count=0;
     
     if (!q)
     {   errprintf("q is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
 
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     for (cur=q->head; cur!=NULL; cur=cur->next)
@@ -215,41 +181,38 @@ s64 queue_count_status(cqueue *q, int status)
     }
     
     assert(pthread_mutex_unlock(&q->mutex)==0);
-    fct_out();
     
     return count;
 }
 
 // add a block at the end of the queue
-s64 queue_add_block(cqueue *q, struct s_blockinfo *blkinfo, int status)
+s64 queue_add_block(cqueue *q, cblockinfo *blkinfo, int status)
 {
-    struct s_queueitem *item;
-    struct s_queueitem *cur;
+    cqueueitem *item;
+    cqueueitem *cur;
     
     if (!q || !blkinfo)
     {   errprintf("a parameter is NULL\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     // create the new item in memory
-    item=malloc(sizeof(struct s_queueitem));
+    item=malloc(sizeof(cqueueitem));
     if (!item)
-    {   errprintf("malloc(%ld) failed: out of memory\n", (long)sizeof(struct s_queueitem));
-        return QERR_NOMEM;
+    {   errprintf("malloc(%ld) failed: out of memory\n", (long)sizeof(cqueueitem));
+        return FSAERR_ENOMEM;
     }
     item->type=QITEM_TYPE_BLOCK;
     item->status=status;
     item->blkinfo=*blkinfo;
     item->next=NULL;
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     // does not make sense to add item on a queue where endofqueue is true
     if (q->endofqueue==true)
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
-        fct_out();
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     }
     
     // wait while (queue-is-full) to let the other threads remove items first
@@ -257,7 +220,6 @@ s64 queue_add_block(cqueue *q, struct s_blockinfo *blkinfo, int status)
     {
         struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     if (q->head==NULL) // if list empty: item is head
@@ -275,18 +237,17 @@ s64 queue_add_block(cqueue *q, struct s_blockinfo *blkinfo, int status)
     
     assert(pthread_mutex_unlock(&q->mutex)==0);
     pthread_cond_broadcast(&q->cond);
-    fct_out();
     
-    return QERR_SUCCESS;
+    return FSAERR_SUCCESS;
 }
 
 s64 queue_add_header(cqueue *q, cdico *d, char *magic, u16 fsid)
 {
-    struct s_headinfo headinfo;
+    cheadinfo headinfo;
     
     if (!q || !d || !magic)
     {   errprintf("parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     memset(&headinfo, 0, sizeof(headinfo));
@@ -297,21 +258,21 @@ s64 queue_add_header(cqueue *q, cdico *d, char *magic, u16 fsid)
     return     queue_add_header_internal(q, &headinfo);
 }
 
-s64 queue_add_header_internal(cqueue *q, struct s_headinfo *headinfo)
+s64 queue_add_header_internal(cqueue *q, cheadinfo *headinfo)
 {
-    struct s_queueitem *item;
-    struct s_queueitem *cur;
+    cqueueitem *item;
+    cqueueitem *cur;
     
     if (!q || !headinfo)
     {   errprintf("parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     // create the new item in memory
-    item=malloc(sizeof(struct s_queueitem));
+    item=malloc(sizeof(cqueueitem));
     if (!item)
-    {   errprintf("malloc(%ld) failed: out of memory 1\n", (long)sizeof(struct s_queueitem));
-        return QERR_NOMEM;
+    {   errprintf("malloc(%ld) failed: out of memory 1\n", (long)sizeof(cqueueitem));
+        return FSAERR_ENOMEM;
     }
     
     item->headinfo=*headinfo;
@@ -319,14 +280,12 @@ s64 queue_add_header_internal(cqueue *q, struct s_headinfo *headinfo)
     item->status=QITEM_STATUS_DONE;
     item->next=NULL;
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     // does not make sense to add item on a queue where endofqueue is true
     if (q->endofqueue==true)
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
-        fct_out();
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     }
     
     // wait while (queue-is-full) to let the other threads remove items first
@@ -334,7 +293,6 @@ s64 queue_add_header_internal(cqueue *q, struct s_headinfo *headinfo)
     {
         struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     item->itemnum=q->curitemnum++;
@@ -350,29 +308,26 @@ s64 queue_add_header_internal(cqueue *q, struct s_headinfo *headinfo)
     q->itemcount++;
     assert(pthread_mutex_unlock(&q->mutex)==0);
     pthread_cond_broadcast(&q->cond);
-    fct_out();
     
-    return QERR_SUCCESS;
+    return FSAERR_SUCCESS;
 }
 
 // function called by the compression thread when a block has been compressed
-s64 queue_replace_block(cqueue *q, s64 itemnum, struct s_blockinfo *blkinfo, int newstatus)
+s64 queue_replace_block(cqueue *q, s64 itemnum, cblockinfo *blkinfo, int newstatus)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     
     if (!q || !blkinfo)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     if (q->head==NULL)
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
         msgprintf(MSG_DEBUG1, "q->head is NULL: list is empty\n");
-        fct_out();
-        return QERR_NOTFOUND; // item not found
+        return FSAERR_ENOENT; // item not found
     }
     
     for (cur=q->head; cur!=NULL; cur=cur->next)
@@ -383,25 +338,23 @@ s64 queue_replace_block(cqueue *q, s64 itemnum, struct s_blockinfo *blkinfo, int
             cur->blkinfo=*blkinfo;
             assert(pthread_mutex_unlock(&q->mutex)==0);
             pthread_cond_broadcast(&q->cond);
-            fct_out();
-            return QERR_SUCCESS;
+            return FSAERR_SUCCESS;
         }
     }
     
     assert(pthread_mutex_unlock(&q->mutex)==0);
-    fct_out();
-    return QERR_NOTFOUND; // not found
+    return FSAERR_ENOENT; // not found
 }
 
 // get number of items to be processed
 s64 queue_count_items_todo(cqueue *q)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     s64 count=0;
     
     if (!q)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     assert(pthread_mutex_lock(&q->mutex)==0);
@@ -418,19 +371,17 @@ s64 queue_count_items_todo(cqueue *q)
 }
 
 // the compression thread requires the first block which has not yet been compressed
-s64 queue_get_first_block_todo(cqueue *q, struct s_blockinfo *blkinfo)
+s64 queue_get_first_block_todo(cqueue *q, cblockinfo *blkinfo)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     s64 itemfound=-1;
     int res;
     
     if (!q || !blkinfo)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
-    fct_in();
-    
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     while (queuelocked_get_end_of_queue(q)==false)
@@ -444,7 +395,6 @@ s64 queue_get_first_block_todo(cqueue *q, struct s_blockinfo *blkinfo)
                 itemfound=cur->itemnum;
                 assert(pthread_mutex_unlock(&q->mutex)==0);
                 pthread_cond_broadcast(&q->cond);
-                fct_out();
                 return itemfound; // ">0" means item found
             }
         }
@@ -452,37 +402,33 @@ s64 queue_get_first_block_todo(cqueue *q, struct s_blockinfo *blkinfo)
         struct timespec t=get_timeout();
         if ((res=pthread_cond_timedwait(&q->cond, &q->mutex, &t))!=0 && res!=ETIMEDOUT)
         {   assert(pthread_mutex_unlock(&q->mutex)==0);
-            fct_out();
-            return QERR_FAIL;
+            return FSAERR_UNKNOWN;
         }
         
-        printwait();
     }
     
     // if it failed at the other end of the queue
     assert(pthread_mutex_unlock(&q->mutex)==0);
-    fct_out();
     
     if (queuelocked_get_end_of_queue(q))
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     else
-        return QERR_FAIL;
+        return FSAERR_UNKNOWN;
 }
 
 // the writer thread requires the first block of the queue if it ready to go
-//s64 queue_dequeue_first(cqueue *q, int *type, char *magic, cdico **dico, struct s_blockinfo *blkinfo)
-s64 queue_dequeue_first(cqueue *q, int *type, struct s_headinfo *headinfo, struct s_blockinfo *blkinfo)
+//s64 queue_dequeue_first(cqueue *q, int *type, char *magic, cdico **dico, cblockinfo *blkinfo)
+s64 queue_dequeue_first(cqueue *q, int *type, cheadinfo *headinfo, cblockinfo *blkinfo)
 {
-    struct s_queueitem *cur=NULL;
+    cqueueitem *cur=NULL;
     s64 itemfound=-1;
     int ret;
     
     if (!q || !headinfo || !blkinfo)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     while (queuelocked_get_end_of_queue(q)==false)
@@ -500,7 +446,6 @@ s64 queue_dequeue_first(cqueue *q, int *type, struct s_headinfo *headinfo, struc
                 q->itemcount--;
                 assert(pthread_mutex_unlock(&q->mutex)==0);
                 pthread_cond_broadcast(&q->cond);
-                fct_out();
                 return itemfound; // ">0" means item found
             }
             else if (cur->type==QITEM_TYPE_HEADER) // item to dequeue is a dico
@@ -513,43 +458,38 @@ s64 queue_dequeue_first(cqueue *q, int *type, struct s_headinfo *headinfo, struc
                 q->itemcount--;
                 assert(pthread_mutex_unlock(&q->mutex)==0);
                 pthread_cond_broadcast(&q->cond);
-                fct_out();
                 return itemfound; // ">0" means item found
             }
             else
             {
                 errprintf("invalid item type in queue\n");
                 assert(pthread_mutex_unlock(&q->mutex)==0);
-                fct_out();
-                return QERR_INVAL;
+                return FSAERR_EINVAL;
             }
         }
         
         struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     // if it failed at the other end of the queue
-    ret=(queuelocked_get_end_of_queue(q)==true)?QERR_ENDOFQUEUE:QERR_FAIL;
+    ret=(queuelocked_get_end_of_queue(q)==true)?FSAERR_ENDOFFILE:FSAERR_UNKNOWN;
     assert(pthread_mutex_unlock(&q->mutex)==0);
-    fct_out();
     
     return ret;
 }
 
 // the extract function wants to read headers from the queue
-s64 queue_dequeue_block(cqueue *q, struct s_blockinfo *blkinfo)
+s64 queue_dequeue_block(cqueue *q, cblockinfo *blkinfo)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     s64 itemnum;
     
     if (!q || !blkinfo)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     // while ((first-item-of-the-queue-is-not-ready) && (not-at-the-end-of-the-queue))
@@ -557,14 +497,12 @@ s64 queue_dequeue_block(cqueue *q, struct s_blockinfo *blkinfo)
     {
         struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     // if it failed at the other end of the queue
     if (queuelocked_get_end_of_queue(q))
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
-        fct_out();
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     }
     
     // should not happen since queuelocked_is_first_block_ready means there is at least one block in the queue
@@ -581,7 +519,6 @@ s64 queue_dequeue_block(cqueue *q, struct s_blockinfo *blkinfo)
         q->itemcount--;
         assert(pthread_mutex_unlock(&q->mutex)==0);
         pthread_cond_broadcast(&q->cond);
-        fct_out();
         return itemnum;
     }
     else
@@ -589,19 +526,18 @@ s64 queue_dequeue_block(cqueue *q, struct s_blockinfo *blkinfo)
         errprintf("dequeue - wrong type of data in the queue: wanted a block, found an header\n");
         assert(pthread_mutex_unlock(&q->mutex)==0);
         pthread_cond_broadcast(&q->cond);
-        fct_out();
-        return QERR_WRONGTYPE;  // ok but not found
+        return FSAERR_WRONGTYPE;  // ok but not found
     }
 }
 
 s64 queue_dequeue_header(cqueue *q, cdico **d, char *magicbuf, u16 *fsid)
 {
-    struct s_headinfo headinfo;
+    cheadinfo headinfo;
     s64 lres;
     
     if (!q || !d || !magicbuf)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     if ((lres=queue_dequeue_header_internal(q, &headinfo))<=0)
@@ -616,17 +552,16 @@ s64 queue_dequeue_header(cqueue *q, cdico **d, char *magicbuf, u16 *fsid)
     return lres;
 }
 
-s64 queue_dequeue_header_internal(cqueue *q, struct s_headinfo *headinfo)
+s64 queue_dequeue_header_internal(cqueue *q, cheadinfo *headinfo)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     s64 itemnum;
     
     if (!q || !headinfo)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     // while ((first-item-of-the-queue-is-not-ready) && (not-at-the-end-of-the-queue))
@@ -634,14 +569,12 @@ s64 queue_dequeue_header_internal(cqueue *q, struct s_headinfo *headinfo)
     {
         struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     // if it failed at the other end of the queue
     if (queuelocked_get_end_of_queue(q))
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
-        fct_out();
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     }
     
     // should not happen since queuelocked_is_first_block_ready means there is at least one block in the queue
@@ -658,27 +591,24 @@ s64 queue_dequeue_header_internal(cqueue *q, struct s_headinfo *headinfo)
             q->itemcount--;
             assert(pthread_mutex_unlock(&q->mutex)==0);
             pthread_cond_broadcast(&q->cond);
-            fct_out();
             return itemnum;
         case QITEM_TYPE_BLOCK:
             errprintf("dequeue - wrong type of data in the queue: expected a dico and found a block\n");
             assert(pthread_mutex_unlock(&q->mutex)==0);
             pthread_cond_broadcast(&q->cond);
-            fct_out();
-            return QERR_WRONGTYPE;  // ok but not found
+            return FSAERR_WRONGTYPE;  // ok but not found
         default: // should never happen
             errprintf("dequeue - wrong type of data in the queue: expected a dico and found an unknown item\n");
             assert(pthread_mutex_unlock(&q->mutex)==0);
             pthread_cond_broadcast(&q->cond);
-            fct_out();
-            return QERR_WRONGTYPE;  // ok but not found
+            return FSAERR_WRONGTYPE;  // ok but not found
     }
 }
 
 // returns true if the first item in queue is a dico or a block which is ready
 bool queuelocked_is_first_item_ready(cqueue *q)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     
     if (!q)
     {   errprintf("a parameter is null\n");
@@ -698,17 +628,16 @@ bool queuelocked_is_first_item_ready(cqueue *q)
 // say what the next item which is ready in the queue is but do not remove it
 s64 queue_check_next_item(cqueue *q, int *type, char *magic)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     
     if (!q || !type || !magic)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
     memset(magic, 0, FSA_SIZEOF_MAGIC);
     *type=0;
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     // while ((first-item-of-the-queue-is-not-ready) && (not-at-the-end-of-the-queue))
@@ -716,14 +645,12 @@ s64 queue_check_next_item(cqueue *q, int *type, char *magic)
     {
         struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     // if it failed at the other end of the queue
     if (queuelocked_get_end_of_queue(q))
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
-        fct_out();
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     }
     
     // test the first item
@@ -734,58 +661,51 @@ s64 queue_check_next_item(cqueue *q, int *type, char *magic)
             *type=cur->type;
             memset(magic, 0, FSA_SIZEOF_MAGIC);
             assert(pthread_mutex_unlock(&q->mutex)==0);
-            fct_out();
-            return QERR_SUCCESS;
+            return FSAERR_SUCCESS;
         }
         else if (cur->type==QITEM_TYPE_HEADER) // item to dequeue is a dico
         {
             memcpy(magic, cur->headinfo.magic, FSA_SIZEOF_MAGIC); // header contents
             *type=cur->type;
             assert(pthread_mutex_unlock(&q->mutex)==0);
-            fct_out();
-            return QERR_SUCCESS;
+            return FSAERR_SUCCESS;
         }
         else
         {
             errprintf("invalid item type in queue: type=%d\n", cur->type);
             assert(pthread_mutex_unlock(&q->mutex)==0);
-            fct_out();
-            return QERR_INVAL;
+            return FSAERR_EINVAL;
         }
     }
     
     assert(pthread_mutex_unlock(&q->mutex)==0);
     pthread_cond_broadcast(&q->cond);
-    fct_out();
     
-    return QERR_NOTFOUND;  // not found
+    return FSAERR_ENOENT;  // not found
 }
 
 // destroy the first item in the queue (similar to dequeue but do not read it)
 s64 queue_destroy_first_item(cqueue *q)
 {
-    struct s_queueitem *cur;
+    cqueueitem *cur;
     
     if (!q)
     {   errprintf("a parameter is null\n");
-        return QERR_INVAL;
+        return FSAERR_EINVAL;
     }
     
-    fct_in();
     assert(pthread_mutex_lock(&q->mutex)==0);
     
     // while ((first-item-of-the-queue-is-not-ready or first-item-is-being-processed-by-comp-thread) && (not-at-the-end-of-the-queue))
     while ( (((cur=q->head)==NULL) || (cur->status==QITEM_STATUS_PROGRESS)) && (queuelocked_get_end_of_queue(q)==false) )
     {   struct timespec t=get_timeout();
         pthread_cond_timedwait(&q->cond, &q->mutex, &t);
-        printwait();
     }
     
     // if it failed at the other end of the queue
     if (queuelocked_get_end_of_queue(q))
     {   assert(pthread_mutex_unlock(&q->mutex)==0);
-        fct_out();
-        return QERR_ENDOFQUEUE;
+        return FSAERR_ENDOFFILE;
     }
     
     // should not happen since queuelocked_is_first_block_ready means there is at least one block in the queue
@@ -807,6 +727,5 @@ s64 queue_destroy_first_item(cqueue *q)
     q->itemcount--;
     assert(pthread_mutex_unlock(&q->mutex)==0);
     pthread_cond_broadcast(&q->cond);
-    fct_out();
-    return QERR_SUCCESS;
+    return FSAERR_SUCCESS;
 }
