@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -15,19 +15,25 @@
  * Homepage: http://www.fsarchiver.org
  */
 
-#include "fsarchiver.h"
-
 #ifndef __WRITEBUF_H__
 #define __WRITEBUF_H__
 
+struct s_dico;
+struct s_blockinfo;
+
+struct s_writebuf;
+typedef struct s_writebuf cwritebuf;
+
 struct s_writebuf
-{
-    char    *data;
-    u64    size;
+{   char *data;
+    u64  size;
 };
 
-struct s_writebuf *writebuf_alloc();
-int writebuf_destroy(struct s_writebuf *wb);
-int writebuf_add_data(struct s_writebuf *wb, void *data, u64 size);
+cwritebuf *writebuf_alloc();
+int writebuf_destroy(cwritebuf *wb);
+int writebuf_add_data(cwritebuf *wb, void *data, u64 size);
+int writebuf_add_dico(cwritebuf *wb, struct s_dico *d, char *magic);
+int writebuf_add_header(cwritebuf *wb, struct s_dico *d, char *magic, u32 archid, u16 fsid);
+int writebuf_add_block(cwritebuf *wb, struct s_blockinfo *blkinfo, u32 archid, u16 fsid);
 
 #endif // __WRITEBUF_H__
